#include <iostream.h>
#include <iomanip.h>
#include <time.h>
#include <stdlib.h>

#include <JTC/JTC.h>

#include "checkboard.h"

static int irand( int a, int e)
{
    double r = e - a + 1;
    return a + (int)(r * rand()/(RAND_MAX+1.0));
}

Checkboard::Checkboard( int size)
 : board_ (new int[size]), size_ (size)
{    
    for (int i = 0; i < size_; ++i) board_[i] = 0; 
}

void Checkboard::entry()
{
    JTCSynchronized synchronized(*this);
    
    int sum, i;

    board_[irand(0, size_-2)] = irand(-1000, 1000);
    // compute sum and put it into last element of board_ 
    for (i = 0, sum = 0; i < size_-1; ++i) {
        JTCThread::currentThread()->sleep( 1); // to force the problem
        sum += board_[i];
    }
    board_[size_-1] = sum;
}

void Checkboard::printAndCheck()
{
    JTCSynchronized synchronized(*this);
    
    int sum, i;

    for (i = 0; i < size_; ++i) {
        if (i % 10 == 0) cout << endl;
        cout << setw(6) << board_[i];
    }

    // compute sum and compare it with last element of board_
    for (i = 0, sum = 0; i < size_-1; ++i)
        sum += board_[i];

    cout << "\n Summe = " << sum << endl;
    if (sum != board_[size_-1]) 
        cerr << " *** Fehler, " << sum << " != " << board_[size_-1] << endl;

}

CheckboardEntryThread::CheckboardEntryThread( Checkboard* cb)
 : cb_ (cb)
{ }

void CheckboardEntryThread::run()
{
    srand( time(0));
    for (;;) {
        cb_ -> entry();
        this->sleep(100);
    }
}

CheckboardPrinterThread::CheckboardPrinterThread( Checkboard* cb)
 : cb_ (cb)
{ }

void CheckboardPrinterThread::run()
{

    for (;;) {
        cb_ -> printAndCheck();
        this->sleep(5000);
    }
}
